#pragma once
#include "IOmgttSensor3_Configs.h"

#include <WiFiClient.h> 
#include <ESP8266WiFi.h>
#include <PubSubClient.h>
#include <ESP8266WebServer.h>
#include <DNSServer.h>
#include <DHT.h>
#include <ESP8266mDNS.h>
#include <WiFiUdp.h>
#include <ArduinoOTA.h>
#include <Adafruit_GFX.h>
#include <Adafruit_SSD1306.h>
#include <Wire.h>
#include "BlueDot_BME280.h"

WiFiClient var_wifiClient;                     // Wifi Client
PubSubClient var_MqttClient(var_wifiClient);   // MQTT Client
ESP8266WebServer var_HttpServer(80);           // HTTP Server
BlueDot_BME280 var_BME280i2c;                  // BME280 Temp, Luftfeuchtigkeit und Barometer
DHT var_dhtSensor(DHTSensorPin, DHTSensorTyp); // DHT



float float_fkt_getTempBME();
float float_fkt_getTempDHT();
float float_fkt_getHumBME();
float float_fkt_getHumDHT();
float float_fkt_getLuftDruckBME();

bool var_IsWifiConnected = false;
bool var_IsMqttConnected = false;
int TMP_int_fkt_LaufZeitSekunden_BerechungMillis =0, TMP_int_fkt_LaufZeitSekunden =0;
String var_SystemSensorName = String(SensorStandort)+String(SensorRaumName)+String(SensorKnoten);


/*
 *  Berechne Laufzeit in Sekunden über einen einfachen Counter und gebe den Zurück
 */

int int_fkt_LaufZeitSekunden(){
  if ( millis()/1000 - TMP_int_fkt_LaufZeitSekunden_BerechungMillis >= 1){                                           // Dieses IF/Else wird genutzt um statt mit millis() die Laufzeit abzufragen und einen Sekundenzähler
    TMP_int_fkt_LaufZeitSekunden = TMP_int_fkt_LaufZeitSekunden+1;                                                   // zu emulieren. Damit soll der Variablenüberlauf lange nach hinten hinausgezögert werden.
    TMP_int_fkt_LaufZeitSekunden_BerechungMillis = millis()/1000;                                                    // Ebenfalls wird das wiederholte Aufrufen von millis() dadurch vermieden.
    delay(2);
 }
 return TMP_int_fkt_LaufZeitSekunden;
}

/*
 *  Berechne Temperatur über mehrere Sensoren, wenn aktiviert!
 */
 
float float_fkt_getTemp(){ 
 int var_SensorCount = 0;
 float var_TemperaturOne = 0;
 float var_TemperaturTwo = 0;
 float var_TemperaturSumme = 0;
 if (BME280i2cNutzen){
   var_SensorCount = var_SensorCount + 1;
   var_TemperaturOne = float_fkt_getTempBME();
 }
 if (DHTSensorNutzen){
  var_SensorCount = var_SensorCount + 1;
  var_TemperaturTwo = float_fkt_getTempDHT();
 }
 var_TemperaturSumme = ((var_TemperaturOne + var_TemperaturTwo)/var_SensorCount);
 return var_TemperaturSumme;
}

/*
 *  Berechne Luftfeuchtigkeit über mehrere Sensoren, wenn aktiviert!
 */
 

float float_fkt_getHum(){ 
 int var_SensorCount = 0;
 float var_LuftfeuchteOne = 0;
 float var_LuftfeuchteTwo = 0;
 float var_LuftfeuchteSumme = 0;
 if (BME280i2cNutzen){
   var_SensorCount = var_SensorCount + 1;
   var_LuftfeuchteOne = float_fkt_getHumBME();
 }
 if (DHTSensorNutzen){
  var_SensorCount = var_SensorCount + 1;
  var_LuftfeuchteTwo = float_fkt_getHumDHT();
 }
 var_LuftfeuchteSumme = ((var_LuftfeuchteOne + var_LuftfeuchteTwo)/var_SensorCount);
 return var_LuftfeuchteSumme;
}



/*
 *  MQTT Publish und Subscribes - Definierung aller Variablen, es werden nur nich immer alle genutzt
 */

String Value101Temperatur;
String Value102Luftfeuchtigkeit;
String Value103Helligkeit;
bool Value104_Bewegung;
String Value105RauchGas;
String Value106MeasureCurrent;
String Value107Barrometer;
String Value108RegenSensor;
String Value301SystemZeit;
String Value302SystemDatum;

/*
 *  MQTT Publish und Subscribes - Definierung aller Topics, es werden nur nich immer alle genutzt
 */
String var_MqttSensorName = String(SensorStandort)+"/"+String(SensorRaumName)+"/"+String(SensorKnoten);

String Value101 = var_MqttSensorName + "/tele/S01_Temperatur";
String Value102 = var_MqttSensorName + "/tele/S02_Luftfeuchtigkeit";
String Value103 = var_MqttSensorName + "/tele/S03_Helligkeit";
String Value104 = var_MqttSensorName + "/tele/S04_Bewegung";
String Value105 = var_MqttSensorName + "/tele/S05_RauchGas";
String Value106 = var_MqttSensorName + "/tele/S06_MeasureCurrent";
String Value107 = var_MqttSensorName + "/tele/S07_Barrometer";
String Value108 = var_MqttSensorName + "/tele/S08_RegenSensor";

String Value201 = var_MqttSensorName + "/stat/S01_IP";
String Value202 = var_MqttSensorName + "/stat/S02_WLAN-Signal";
String Value203 = var_MqttSensorName + "/stat/S03_TimeStamp";
String Value204 = var_MqttSensorName + "/stat/S04_Uptime";
String Value205 = var_MqttSensorName + "/stat/S05_AkkuStand";
String Value206 = var_MqttSensorName + "/stat/S06_Datum";

String Value301 = "info/SystemZeit";
String Value302 = "info/SystemDatum";
String Value303 = var_MqttSensorName + "/cmnd/A01_Nachtlicht";
String Value304 = var_MqttSensorName + "/cmnd/A02_Signal";

